@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -26,6 +27,23 @@
 
 #include "gmime-stream-filter.h"
 
+
+/**
+ * SECTION: gmime-stream-filter
+ * @title: GMimeStreamFilter
+ * @short_description: A filtering stream
+ * @see_also: #Filters
+ *
+ * A #GMimeStream meant for filtering data passing through it.
+ *
+ * This stream class is useful for converting data of one type to
+ * another using #GMimeFilter objects.
+ *
+ * When data passes through a #GMimeStreamFilter, it will pass through
+ * #GMimeFilter filters in the order they were added.
+ **/
+
+
 #define READ_PAD (64)		/* bytes padded before buffer */
 #define READ_SIZE (4096)
 
@@ -84,7 +102,7 @@ g_mime_stream_filter_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeStreamFilter),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_stream_filter_init,
 		};
 		
@@ -234,7 +252,7 @@ stream_write (GMimeStream *stream, const char *buf, size_t n)
 		f = f->next;
 	}
 	
-	if (g_mime_stream_write (filter->source, buffer, n) != n)
+	if (g_mime_stream_write (filter->source, buffer, n) == -1)
 		return -1;
 	
 	/* return original input len because that's what our caller expects */
@@ -306,6 +324,9 @@ stream_reset (GMimeStream *stream)
 	struct _GMimeStreamFilterPrivate *priv = filter->priv;
 	struct _filter *f;
 	
+	if (g_mime_stream_reset (filter->source) == -1)
+		return -1;
+	
 	priv->filteredlen = 0;
 	priv->flushed = FALSE;
 	
@@ -316,7 +337,7 @@ stream_reset (GMimeStream *stream)
 		f = f->next;
 	}
 	
-	return g_mime_stream_reset (filter->source);
+	return 0;
 }
 
 static off_t
@@ -343,7 +364,7 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
 	GMimeStreamFilter *filter = (GMimeStreamFilter *) stream;
 	GMimeStreamFilter *sub;
 	
-	sub = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL, NULL);
+	sub = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL);
 	sub->source = filter->source;
 	g_object_ref (sub->source);
 	
@@ -371,9 +392,7 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
 		sub->priv->filterid = filter->priv->filterid;
 	}
 	
-	g_mime_stream_construct (GMIME_STREAM (filter),
-				 sub->source->bound_start,
-				 sub->source->bound_end);
+	g_mime_stream_construct (GMIME_STREAM (filter), start, end);
 	
 	return GMIME_STREAM (sub);
 }
@@ -395,7 +414,7 @@ g_mime_stream_filter_new_with_stream (GMimeStream *stream)
 	
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), NULL);
 	
-	filter = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL, NULL);
+	filter = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL);
 	filter->source = stream;
 	g_object_ref (stream);
 	
